/*
 * @(#)JaveMovieFile.java  1.2  23rd December 2001 Markus Gebhard
 * @(#)JaveMovieFile.java  1.1  11th July 2001 Markus Gebhard
 * @(#)JaveMovieFile.java  1.0  28th June 2001 Markus Gebhard
 * @(#)JaveMovieFile.java  0.1  10th June 2001 Markus Gebhard
 * 
 * This Software is Open Source. You are allowed to change anything,
 * but there are some limitations:
 *
 *  - You must always include the source code
 *  - You are not allowed to remove this Header from the source code
 *  - You have to add a link to http://www.jave.de so people know
 *    where to find the original version and information about it
 *  - You have to let me know about new releases
 *
 *                                      Markus Gebhard  markus@jave.de
 */
 
 
/**
The static methode in this class can be used  to load a Jave-movie file.
A Jave-movie file contains several compressed frames (JaveMovieFrame).
*/
 
import java.io.*;
import java.util.Vector;
import java.awt.Color;
//

public class JaveMovieFile{
  /** Name of this file */
  protected String fileName;

  /** All frames int this movie file in the order of appearance */
  protected Vector frames;
   
  /** Name of the author of this file (or null) */
  protected String authorName;

  /** Email address of the author (or null) */
  protected String authorEmail;
  
  /** Information about the software that created this file (or null) */
  protected String software;

  /** The date when this file was created (or null) */
  protected String date;

  /** Default duration of a frame in milliseconds */
  protected int defaultDuration;
  
  protected boolean modified;
 
  
/**
* Creates a new Jave Movie File with the name 'fileName'.
* Must be loaded by calling load() before use!
*/
public JaveMovieFile(String fileName){
  this.fileName=fileName;
  this.defaultDuration=JavePlayer.DEFAULT_DURATION;
  modified=false;
}

/**
* Creates a new empty Jave Movie File with the name 'fileName'.
*/
public JaveMovieFile(){
  this.fileName=null;
  this.defaultDuration=JavePlayer.DEFAULT_DURATION;

  frames=new Vector(200);
  modified=false;
}

/**
* Default duration of a frame in milliseconds
* (was 66ms if not specified here).
*
* Has to be set before this file is loaded!
*/
public void setDefaultDuration(int ms){
  this.defaultDuration=ms;
}

public int getDefaultDuration(){
  return defaultDuration;
}


public String getDate(){
  return date;
}

public void setDate(String date){
  //No change?
  if (isEquivalent(date, this.date))
    return;

  this.date=date;
  modified=true;
}

public String getSoftware(){
  return software;
}

public void setSoftware(String software){
  //No change?
  if (isEquivalent(software, this.software))
    return;

  this.software=software;
  modified=true;
}

public String getAuthor(){
  return authorName;
}

public void setAuthor(String authorName){
  //No change?
  if (isEquivalent(authorName, this.authorName))
    return;

  this.authorName=authorName;
  modified=true;
}

public String getEMail(){
  return authorEmail;
}


public void setEMail(String authorEmail){
  //No change?
  if (isEquivalent(authorEmail, this.authorEmail))
    return;

  this.authorEmail=authorEmail;
  modified=true;
}



protected final static boolean isEquivalent(String a, String b){
  if (a==null || a.length()==0){
    return (b==null || b.length()==0);
  }else{
    return a.equals(b);
  }
}





public void load() throws Exception{
  frames=new Vector();  
  try{
    BufferedReader reader=ZIPTool.openPossiblyZipped(ZIPTool.toUrl(fileName));
    if (reader==null)
      throw new IOException("Unable to open file "+fileName);
  
    int frameCount=0;
    int lastDuration=defaultDuration;
    Color foreground=Color.black;
    Color background=Color.white;
    
    JaveMovieFrame currentFrame=null;
    
    //Remember content from last frame for dublicate frame contents
    String lastContent=null;
    String s;
    while ((s=reader.readLine())!=null){
      
      if (s.startsWith("J:")){
        if (currentFrame!=null)
          frames.addElement(currentFrame);
        
        currentFrame=new JaveMovieFrame();
        currentFrame.setDuration(lastDuration);
        currentFrame.setForeground(foreground);
        currentFrame.setBackground(background);
        
        ++frameCount;

        //J: without CODE following => repeat previous content
        if (s.length()==2){
          currentFrame.setContent(lastContent);
          continue;
        }
        
        //There is an old format for this command - it is still supported for compatibility reasons:
        //Old format: J:SCROLL_X SCROLL_Y CURSOR_X CURSOR_Y CODE
        //It can be recognized by having a ciffer as the first character
        char ch=s.charAt(2);
        if (ch>='0' && ch<='9'){
          //Old format
          try{
            int i1=2;
            int i2=s.indexOf(' ',i1+1);
            int i3=s.indexOf(' ',i2+1);
            int i4=s.indexOf(' ',i3+1);
            int i5=s.indexOf(' ',i4+1);
            
            String s1=s.substring(i1,   i2);
            String s2=s.substring(i2+1, i3);
            String s3=s.substring(i3+1, i4);
                        
            String s4=null;
            if (i5!=-1)
              s4=s.substring(i4+1, i5);
            else
              s4=s.substring(i4+1);
            
            int scrollX=Integer.parseInt(s1);
            int scrollY=Integer.parseInt(s2);
            int cursorX=Integer.parseInt(s3);
            int cursorY=Integer.parseInt(s4);
  
            String content=null;
            if (i5==-1)
              //The content is the same as in the frame before
              content=lastContent;
            else
              content=s.substring(i5+1);
  
            //Remember content from last frame for dublicate frame contents
            lastContent=content;
             
            currentFrame.setContent(content);
            currentFrame.setScrollX(scrollX);
            currentFrame.setScrollY(scrollY);
            currentFrame.setCursorX(cursorX);
            currentFrame.setCursorY(cursorY);
            
          }catch (Exception e){
            System.err.println("Warning: Error in format of Jave Movie."+e);  
          }
        }else{
          //New format as in specification 1.0:
          String content=s.substring(2);
  
          //Remember content from last frame for dublicate frame contents
          lastContent=content;
             
          currentFrame.setContent(content);
        }
      }else
      if (s.startsWith("S:")){
        try{
          int i1=2;
          int i2=s.indexOf(' ',i1+1);
          int i3=s.indexOf(' ',i2+1);
          String s1=s.substring(i1,   i2);
          String s2=s.substring(i2+1, i3);
          String content=s.substring(i3+1);
          int selectionX=Integer.parseInt(s1);
          int selectionY=Integer.parseInt(s2);
        
          currentFrame.setSelection(content);
          currentFrame.setSelectionX(selectionX);
          currentFrame.setSelectionY(selectionY);
        }catch (Exception e){
          System.err.println("Warning: Error in format of Jave Movie.");  
        }
      }else
      if (s.startsWith("T:")){
        String toolName=s.substring(2);
        currentFrame.setTool(toolName);
      }else
      if (s.startsWith("A:")){
        String actionName=s.substring(2);
        currentFrame.setAction(actionName);
      }else
      if (s.startsWith("+:")){
        try{
          int duration=Integer.parseInt(s.substring(2));
          currentFrame.setDuration(duration);
          lastDuration=duration;
        }catch (NumberFormatException e){
          System.err.println("Warning: Error in format of Jave Movie:  "+s);  
        }
      }else
      if (s.startsWith("|:")){
        //Cursor location |:CURSOR_X CURSOR_Y
        int i1=s.indexOf(' ',2);
        try{
          int cursorX=Integer.parseInt(s.substring(2,i1)); 
          int cursorY=Integer.parseInt(s.substring(i1+1)); 
          currentFrame.setCursorX(cursorX);
          currentFrame.setCursorY(cursorY);
        }catch (Exception e){
          System.err.println("Warning: Error in format of Jave Movie: "+s);     
        }
      }else
      if (s.startsWith("^:")){
        //Scroll location ^:SCROLL_X SCROLL_Y
        int i1=s.indexOf(' ',2);
        try{
          int scrollX=Integer.parseInt(s.substring(2,i1)); 
          int scrollY=Integer.parseInt(s.substring(i1+1)); 
          currentFrame.setScrollX(scrollX);
          currentFrame.setScrollY(scrollY);
        }catch (Exception e){
          System.err.println("Warning: Error in format of Jave Movie: "+s);     
        }
      }else
      if (s.startsWith("@:")){
        if (authorEmail==null && s.length()>2)
          authorEmail=s.substring(2);
      }else
      if (s.startsWith("N:")){
        if (authorName==null && s.length()>2)
          authorName=s.substring(2);
      }else
      if (s.startsWith("D:")){
        if (date==null && s.length()>2)
          date=s.substring(2);
      }else
      if (s.startsWith("C:")){
// System.err.println(s);       
        try{
          int i1=s.indexOf(' ',2);
          background=JavePlayerTools.hexToColor(s.substring(2,i1));
          foreground=JavePlayerTools.hexToColor(s.substring(i1+1));
          if (currentFrame!=null){
            currentFrame.setForeground(foreground);
            currentFrame.setBackground(background);
          }
// System.err.println(foreground);
// System.err.println(background);
        }catch (Exception e){
          System.err.println("Warning: Error in format of Jave Movie: "+s);     
        }
      }else
      if (s.startsWith("*:")){
        if (software==null && s.length()>2)
          software=s.substring(2);
      }else
      if (s.startsWith("#")){
        currentFrame.setSoundTrigger(true);
      }
    }
   
    if (currentFrame!=null)
      frames.addElement(currentFrame);

  }catch (FileNotFoundException e){
throw new Exception("Error loading Jave movie: File not found: '"+fileName+"'");  
  }catch (IOException e){
throw new Exception("Error loading Jave movie: Wrong file format.");  
  }
  
  modified=false;
}

public void add(JaveMovieFrame newFrame){
  frames.addElement(newFrame);
  modified=true;
}

public void insertFrame(JaveMovieFrame newFrame, int index){
  frames.insertElementAt(newFrame, index);
  modified=true;
}

public void deleteFrame(int index){
  frames.removeElementAt(index);
  modified=true;
}

public void moveFrameRight(int index){
  Object f=frames.elementAt(index);
  frames.removeElementAt(index);
  frames.insertElementAt(f, index+1);
  modified=true;
}

public void moveFrameLeft(int index){
  moveFrameRight(index-1);
}

public void setModified(boolean what){
  modified=what;
}

public boolean isModified(){
  return this.modified;
}

public void setFileName(String fileName){
  this.fileName=fileName;
  modified=true;
}

public String getFileName(){
  return this.fileName;
}

public void save() throws Exception{
  try{
    BufferedWriter writer=new BufferedWriter(new FileWriter(fileName));

    int lastDuration=defaultDuration;
    int lastCursorX=-1;
    int lastCursorY=-1;
    int lastScrollX=-1;
    int lastScrollY=-1;
    Color lastForeground=null;
    Color lastBackground=null;

    //Software information
    if (software!=null){
      writer.write("*:");
      writer.write(software.trim());
      writer.newLine();
    }
      
    //Date
    if (date!=null){
      writer.write("D:");
      writer.write(date.trim());
      writer.newLine();
    }
    
    //Author name
    if (authorName!=null){
      writer.write("N:");
      writer.write(authorName.trim());
      writer.newLine();
    }
    
    //Author name
    if (authorEmail!=null){
      writer.write("@:");
      writer.write(authorEmail.trim());
      writer.newLine();
    }
    
    
    
    for (int i=0;i<frames.size();++i){
      JaveMovieFrame currentFrame=(JaveMovieFrame)frames.elementAt(i);

      //Compress content as good as possible and write it to the file
      char[][] content=currentFrame.getContent();
      writer.write("J:");
      writer.write(AsciiPacker.encodeOptimized(content));
      writer.newLine();

      //Selection if any
      char[][] selection=currentFrame.getSelection();
      if (selection!=null){
        writer.write("S:");
        writer.write(String.valueOf(currentFrame.getSelectionX()));
        writer.write(" ");
        writer.write(String.valueOf(currentFrame.getSelectionY()));
        writer.write(" ");
        writer.write(AsciiPacker.encodeOptimized(selection));
        writer.newLine();
      }
      
      //Cursor coordinates if changed
      int cursorX=currentFrame.getCursorX();
      int cursorY=currentFrame.getCursorY();
      if (cursorX!=lastCursorX || cursorY!=lastCursorY){
        writer.write("|:");
        writer.write(String.valueOf(cursorX));
        writer.write(" ");
        writer.write(String.valueOf(cursorY));
        writer.newLine();
        lastCursorX=cursorX;
        lastCursorY=cursorY;
      }
      
      //Scroll coordinates if changed
      int scrollX=currentFrame.getScrollX();
      int scrollY=currentFrame.getScrollY();
      if (scrollX!=lastScrollX || scrollY!=lastScrollY){
        writer.write("^:");
        writer.write(String.valueOf(scrollX));
        writer.write(" ");
        writer.write(String.valueOf(scrollY));
        writer.newLine();
        lastScrollX=scrollX;
        lastScrollY=scrollY;
      }
      
      //Duration if changed
      int duration=currentFrame.getDuration();
      if (duration!=lastDuration){
        writer.write("+:");
        writer.write(String.valueOf(duration));
        writer.newLine();
        lastDuration=duration;
      }
      
      //Color if changed
      Color fg=currentFrame.getForeground();
      Color bg=currentFrame.getBackground();
      if (!fg.equals(lastForeground) || !bg.equals(lastBackground)){
        writer.write("C:");
        writer.write(JavePlayerTools.colorToHex(bg));
        writer.write(" ");
        writer.write(JavePlayerTools.colorToHex(fg));
        writer.newLine();
        lastForeground=fg;
        lastBackground=bg;
      }
    
      //Tool if any
      String tool=currentFrame.getTool();
      if (tool!=null){
        writer.write("T:");
        writer.write(tool.trim());
        writer.newLine();
      }
      
      //Action if any
      String action=currentFrame.getAction();
      if (action!=null){
        writer.write("A:");
        writer.write(action.trim());
        writer.newLine();
      }
    }
    writer.close();
    modified=false;
  
  }catch (IOException e){
throw new Exception("Error saving Jave movie: "+e);  
  }
}

public JaveMovieFrame getFrame(int index){
  return (JaveMovieFrame)frames.elementAt(index);  
}


public int getFrameCount(){
  if (frames==null)
    return 0;
    
  return frames.size();
}


}