/*
 * @(#)JavePlate.java  1.2  23rd December 2001 Markus Gebhard
 * @(#)JavePlate.java  1.1  11th July 2001 Markus Gebhard
 * @(#)JavePlate.java  1.0  28th June 2001 Markus Gebhard
 * @(#)JavePlate.java  0.1  10th June 2001 Markus Gebhard
 * 
 * This Software is Open Source. You are allowed to change anything,
 * but there are some limitations:
 *
 *  - You must always include the source code
 *  - You are not allowed to remove this Header from the source code
 *  - You have to add a link to http://www.jave.de so people know
 *    where to find the original version and information about it
 *  - You have to let me know about new releases
 *
 *                                      Markus Gebhard  markus@jave.de
 */

import java.awt.*;

/**
This class is intended for displaying a JaveFrame.

By calling setContent(JaveFrame f) you set the content that has
to be displayed. The frame will be unpacked within this methode.
*/
public class JavePlate extends Canvas{
  /** The UNCOMPRESSED content */
  protected char[][] content;
  /** The UNCOMPRESSED selection (if any) */
  protected char[][] selection;

  protected JaveMovieFrame currentFrame;
  
  /** Width in characters */
  protected int width;
  /** Height in characters */
  protected int height;
  /** Width of the selection (if any) in characters */
  protected int selectionWidth;
  /** Height of the selection (if any) in characters */
  protected int selectionHeight;

  /** The width of a single character */
  protected int charWidth;
  /** The height of a single character */
  protected int charHeight;

  /** Flag that shows whether or not this plate has been repainted since last
  call to repaint() */
  protected boolean isRepainted;

  protected String statusMessage="* No Movie Loaded *";
  
  protected final static Font MESSAGE_FONT=new Font("Dialog", Font.PLAIN, 11);

  
  protected final static int DEFAULT_FONT_SIZE = 11;
  protected final static int MIN_FONT_SIZE     =  7;
  protected final static int MAX_FONT_SIZE     = 16;
  
  protected int fontSize;
 
  
  /** for double buffering */
  private Dimension    offDimension;
  /** for double buffering */
  private Image        offImage;
  /** for double buffering */
  private Graphics     offGraphics;
  
  
public JavePlate(){
  setFontSize(DEFAULT_FONT_SIZE);
}
  
  
public void setStatusMessage(String statusMessage){
  this.statusMessage=statusMessage;
  repaint();
}
  
public void setContent(JaveMovieFrame frame){
  if (frame==null){
    this.content=null;
    this.selection=null;
    this.currentFrame=frame;
    return;
  }
  
  
  
  
  
  //Uncompress the content and selection and determine their dimensions
  this.content   = frame.getContent();
  this.selection = frame.getSelection();
  
  this.height = content.length;
  this.width  = content[0].length;

  if (selection!=null){
    this.selectionHeight = selection.length;
    this.selectionWidth  = selection[0].length;
  }
  
  Color fg=frame.getForeground();
  if (fg!=null){
    setForeground(fg);
  }
  Color bg=frame.getBackground();
  if (bg!=null){
    setBackground(bg);
  }
  
  this.currentFrame=frame;
}

public Dimension getPreferredSize(){
  return new Dimension(width*charWidth+4, height*charHeight+4);
}
public Dimension getMinimumSize(){
  return getPreferredSize();
}
public Dimension getMaximumSize(){
  return getPreferredSize();
}

protected void setFontSize(int size){ 
  if (size==fontSize)
    return;
    
  fontSize=size;
  setFont(new Font("Monospaced", Font.PLAIN, fontSize));
 
  /* There is a problem with font sizes in some Java VMs. At least
  the height of a Font differs a lot from JRE to JRE...
  So this is how it is done here (and in Jave):
    1) The width of the characters is metered using FontMetrics.
      This has to be done this way, because a line
      of text is being displayed as a single String.
    2) The height of the characters (/line) is computed by using
      the fontSize. I use a formula I have found out by making
      some tests at a big rangle of font sizes. It is quadratic.
  */
  
  
  // 1)
  charWidth=getFontMetrics(getFont()).stringWidth("#");
  // 2)
  charHeight=(int)Math.round( -0.14*fontSize*fontSize + 4.56*fontSize -19.3);

  //
  //
  //     MISSING: tell parent container to adjust size!
  //PARENT.doLayout()
  //
  
  repaint();
}




public void zoomIn(){
  if (fontSize<MAX_FONT_SIZE)
    setFontSize(fontSize+1);
}

public void zoomOut(){
  if (fontSize>MIN_FONT_SIZE)
    setFontSize(fontSize-1);
}

public void paint(Graphics g){
  update(g);
}

public void update(Graphics g){
  Dimension d=getSize();

  //Create new buffer for painting if old buffer invalid
  if ( (offGraphics == null) ||
       (offDimension.width != d.width)||
       (offDimension.height!= d.height)) {
    offImage     = createImage(d.width, d.height);
    offGraphics  = offImage.getGraphics();
    offDimension = new Dimension(d.width, d.height);
  }
  

  //clear Background  
  offGraphics.setColor(getBackground());
  offGraphics.fillRect(0,0,d.width,d.height);
  
  //Draw a nice 3d-Border
  offGraphics.setColor(SystemColor.controlDkShadow);
   offGraphics.drawLine(1,1, d.width-1,1);   //north bottom
   offGraphics.drawLine(1,2, 1, d.height-1); //west right
  offGraphics.setColor(SystemColor.controlShadow);
   offGraphics.drawLine(0,0, d.width-1,0);   //north top
   offGraphics.drawLine(0,1, 0, d.height-1); //west left
  offGraphics.setColor(SystemColor.controlHighlight);
   offGraphics.drawLine(1,d.height-2, d.width-2 ,d.height-2); //south top
   offGraphics.drawLine(d.width-2 ,d.height-3, d.width-2, 1); //east left
  offGraphics.setColor(SystemColor.controlLtHighlight);
   offGraphics.drawLine(0,d.height-1, d.width-1 ,d.height-1); //south bottom
   offGraphics.drawLine(d.width-1 ,d.height-2, d.width-1, 0); //east right

   
  offGraphics.setColor(getForeground());
  offGraphics.setFont(getFont());  

  //Paint the content
  if (content!=null && content.length>0){
    //Origin (content will be displayed centered!)
    int x0=(d.width-content[0].length*charWidth)/2;
    int y0=(d.height-content.length*charHeight)/2;
 
    //Top left corner should never be cut
    if (x0<2)
      x0=2;
    if (y0<2)
      y0=2;
    
    
    
    for (int y=0;y<height;++y){
      offGraphics.drawString(new String(content[y]),
                             x0,
                             y0+y*charHeight+ charHeight*3/4);
    }
        
    //Paint selection (if any)
    if (selection!=null){
      Color fg=getForeground();
      Color bg=getBackground();
      
      offGraphics.setColor(new Color( (fg.getRed()  +bg.getRed())  /2,
                                      (fg.getGreen()+bg.getGreen())/2,
                                      (fg.getBlue() +bg.getBlue()) /2 ));
                                      
                                      
      int selectionX=currentFrame.getSelectionX();
      int selectionY=currentFrame.getSelectionY();
      
      
      for (int y=0;y<selectionHeight;++y){
        offGraphics.drawString(new String(selection[y]),
                               x0+selectionX*charWidth,
                               y0+(selectionY+y)*charHeight+ charHeight*3/4);
      }

      
      
      
      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
      
      //Paint a border for the selection
      offGraphics.setColor(getBackground().darker());
      offGraphics.drawRect(x0+selectionX*charWidth,
                           y0+selectionY*charHeight,
                           selectionWidth*charWidth,
                           selectionHeight*charHeight);
    }

//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
//TODO: Optional!!!!      
    
    
    //Paint the cursor if the current tool is the text tool
    String tool=currentFrame.getTool();
    int cursorX=currentFrame.getCursorX();
    int cursorY=currentFrame.getCursorY();
    if (tool!=null && tool.equals("Text") && cursorX>=0 && cursorY>=0){
      //Paint text cursor
      offGraphics.setColor(Color.gray);
      offGraphics.drawLine(x0+cursorX*charWidth+2,
                           y0+cursorY*charHeight+charHeight-2,
                           x0+cursorX*charWidth+2,
                           y0+cursorY*charHeight+2);
    }
  }else{
    //No content => display status Message if any
    if (statusMessage!=null && statusMessage.length()>0){
      offGraphics.setFont(MESSAGE_FONT);
      int w=getFontMetrics(MESSAGE_FONT).stringWidth(statusMessage);
      offGraphics.drawString(statusMessage, (d.width-w)/2, d.height/2+12);
    }
  }
   
  //Display buffer
  g.drawImage(offImage, 0, 0, this);
  isRepainted=true;
}

public void repaint(){
  isRepainted=false;
  super.repaint();  
}
  
public boolean isRepainted(){
  return isRepainted;
}
  
}