/*
 * @(#)JavePlayer.java  1.2  23rd December 2001 Markus Gebhard
 * @(#)JavePlayer.java  1.1   9th September 2001 Markus Gebhard
 *   (fixed bug of durationmultiplicator initialized to 0.0 when
 *   no control component)
 * @(#)JavePlayer.java  1.0  28th June 2001 Markus Gebhard
 * @(#)JavePlayer.java  0.1  10th June 2001 Markus Gebhard
 * 
 * This Software is Open Source. You are allowed to change anything,
 * but there are some limitations:
 *
 *  - You must always include the source code
 *  - You are not allowed to remove this Header from the source code
 *  - You have to add a link to http://www.jave.de so people know
 *    where to find the original version and information about it
 *  - You have to let me know about new releases
 *
 *                                      Markus Gebhard  markus@jave.de
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.AudioClip;


/**
This class is a complete player for Jav-movie files.
The Panel defined by JavePlayer can be used in an Applet
(JavePlayerApplet is an Applet using this Panel) of a frame.

This class is also the main class when using the application.
*/

public class JavePlayer extends Panel
                        implements ItemListener,
                                   WindowListener,
                                   ActionListener,
                                   MouseListener,
                                   Runnable{
  protected     JavePlate plate;
  protected JaveMovieFile movieFile;
  /** Backgound sound - only if attached manually or by applet parameter
  null otherwise.*/
  protected     AudioClip audioClip;
  /** When and how to play the audioclip (if attached) */
  protected           int audioStart;
  
  protected           int currentFrameIndex;
  
  protected final static String TITLE="Jave Player V1.2";

  //When and how to play the audioclip (if attached):
  public final static int     SOUND_ONCE  = 0;
  public final static int     SOUND_LOOP  = 1;
  public final static int     SOUND_EVENT = 2;

  public final static int     DEFAULT_DURATION    = 66;
  public final static int     DEFAULT_CONTROLS    =  2;
  public final static boolean DEFAULT_LOOP        =  false;
  public final static boolean DEFAULT_AUTOSTART   =  false;
  public final static Color   DEFAULT_BACKGROUND  = Color.white;
  public final static Color   DEFAULT_FOREGROUND  = Color.black;
  public final static int     DEFAULT_SOUND_START = SOUND_ONCE;
  
  protected Button bPrev,bPlay,bPause,bStop,bNext, bZoomIn, bZoomOut;  
  protected TextField tfTool,tfAction,tfFrame;
  protected Choice chSpeed;
  protected Checkbox cbReverse;
  protected Label lFps;
  
  protected boolean reverse=false;
  
  protected final static String[] STR_SPEEDS={
    "Slowest (0.1x)",
    "Slower (0.2x)",
    "Slow (0.5x)",
    "Normal Speed (1x)",
    "Fast (1.3x)",
    "Faster (2x)",
    "Fastest (4x)",
    "As fast as possible"
  };
  
  protected final static double[] DURATION_MULTIPLICATOR={
    10.0, //""
    5.0,  //""
    2.0,  //""
    1.0,  //"normal"
    0.75, //""
    0.5,  //""
    0.25, //""
    0.0   //"As fast as possible"
  };
  
  protected final static int DEFAULT_SPEED=3; //normal
  protected int sleepTime; //Duration of the current frame
  protected double durationMultiplicator=1.0;
  
  protected Thread thread;

  protected boolean pause;
  protected boolean shallStop;
   
  protected int defaultDuration;
  protected boolean loop;
  protected boolean autostart;
  
  
  
public JavePlayer(){
  this(DEFAULT_DURATION,
       DEFAULT_CONTROLS,
       DEFAULT_LOOP,
       DEFAULT_AUTOSTART,
       DEFAULT_BACKGROUND,
       DEFAULT_FOREGROUND);
}
  
public JavePlayer(int controls){
  this(DEFAULT_DURATION,
       controls,
       DEFAULT_LOOP,
       DEFAULT_AUTOSTART,
       DEFAULT_BACKGROUND,
       DEFAULT_FOREGROUND);
}
  
  
public JavePlayer(int duration, int controls, boolean loop, boolean autostart,
                  Color background, Color foreground){
  super();

  this.defaultDuration=duration;
  this.loop=loop;
  this.autostart=autostart;

  setLayout(new BorderLayout(2,2));
  
  /* controls==-1 => no player output window. This is neccessary for the 
  * JMOV Editor in JavE!
  */
  if (controls!=-1){
    plate=new JavePlate();
    plate.setBackground(background);
    plate.setForeground(foreground);
    plate.addMouseListener(this);
    add(plate,  BorderLayout.CENTER);
  }
   
  currentFrameIndex=0;

  switch(controls){
    case 0:
      //No controls => finished
    break;
    case 1:
      //Just the controlBar with buttons
      add(createControlBar(),  BorderLayout.SOUTH);
    break;
    default:
      //All controls
      Panel p=new Panel();
      p.setLayout(new GridLayout(0,1,2,2));
      p.add(createStatusBar());
      p.add(createControlBar());
      add(p, BorderLayout.SOUTH);
    break;
  }
}


public void setAudioClip(AudioClip audioClip){
  this.audioClip=audioClip;
}

public void setAudioStart(int audioStart){
  this.audioStart=audioStart;
}



public void setPlateBackground(Color bgColor){
  if (plate!=null)
    plate.setBackground(bgColor);
}
public void setPlateForeground(Color fgColor){
  if (plate!=null)
    plate.setForeground(fgColor);
}

public void load(String fileName){
  try{
    movieFile=new JaveMovieFile(fileName);
    movieFile.setDefaultDuration(defaultDuration);
    movieFile.load();
  }catch (Exception e){
    if (plate!=null){
      plate.setStatusMessage(e.getMessage());
      plate.setContent(null);
    }
    return;
  }
 
  if (plate!=null)
    if (movieFile.getFrameCount()>0)
      plate.setContent(movieFile.getFrame(0));
    else
      plate.setContent(null);
    
  if (autostart)
    doPlay();
}


public void run(){
  long timeStart=System.currentTimeMillis();
  long wait;
  pause=false;
  
//long time1= System.currentTimeMillis();
  int fpsCounter=0;
  long fpsTimerStart=System.currentTimeMillis();
  
  while (!shallStop
         && (loop || ((!reverse && currentFrameIndex<movieFile.getFrameCount()-1)
                     || (reverse && currentFrameIndex>0)))      ){
    //Wait for the selected period minus the time already elapsed
    wait=(long)(sleepTime*durationMultiplicator)-(System.currentTimeMillis()-timeStart);
    if (wait>0){
      try{
        Thread.currentThread().sleep(wait);
      }catch(InterruptedException e){
      }
    }

    ++fpsCounter;  
    if ((fpsCounter%10)==0){
      long delta=System.currentTimeMillis()-fpsTimerStart;
      fpsTimerStart=System.currentTimeMillis();
    
      double fps=10000.0/(double)delta;
      if (lFps!=null)
        lFps.setText(String.valueOf((double)((int)(fps*10))/10.0)+" fps");//  System.err.println(fps);
    }
        
    //Start timer
    timeStart=System.currentTimeMillis();
    //Do not paint the next frame before the current one has been displayed
    while(plate!=null && !plate.isRepainted())
      Thread.currentThread().yield();
      
    if (!pause && !shallStop)
      doNext();
      
    //Synchronize graphics
    if (plate!=null)
      plate.getToolkit().sync();
  }

  thread=null;
//long time0= System.currentTimeMillis();
//System.err.println((time0-time1)+"ms");
}


public void setMovieFile(JaveMovieFile f){
  doStop();
  
  movieFile=f;
  currentFrameIndex=-1;
  setFrame(0);
}


protected void setFrame(int index){
  if (index==currentFrameIndex)
    return;
  
  currentFrameIndex=index;
 
  if (currentFrameIndex<0 || currentFrameIndex>=movieFile.getFrameCount()){
    if (plate!=null && movieFile.getFrameCount()==0)
      plate.setStatusMessage("* Empty Movie *");

    if (plate!=null){
      plate.setContent(null);
      plate.repaint();
    }
    tfFrame.setText("");
    tfTool.setText("");
    tfAction.setText("");
    return;
  }
  
  JaveMovieFrame currentFrame=movieFile.getFrame(currentFrameIndex);
  sleepTime=currentFrame.getDuration();
  if (plate!=null){
    plate.setContent(currentFrame);
    plate.repaint();
  }
  
  
  if (tfFrame!=null)
    tfFrame.setText((currentFrameIndex+1)+"/"+movieFile.getFrameCount());
  if (tfTool!=null)
    tfTool.setText(currentFrame.getTool());
  if (tfAction!=null)
    tfAction.setText(currentFrame.getAction());
}

public void doPlay(){
  if (reverse)
    currentFrameIndex=movieFile.getFrameCount()-1;
  else
    currentFrameIndex=0;

  setFrame(currentFrameIndex);
  
  if (thread!=null){
    shallStop=true;
    try{
      thread.interrupt();
      
      while (thread!=null || thread.isAlive())
        Thread.yield();
    }catch (SecurityException e){
    }
  }
  
  thread=new Thread(this);
  thread.setPriority(Thread.MIN_PRIORITY);
  shallStop=false;
  
  
  //Start sound if available and start options fit
//  System.err.println(audioClip+" "+audioStart);
  
  if (audioClip!=null){
    if (audioStart==SOUND_ONCE)
      audioClip.play();
    else
    if (audioStart==SOUND_LOOP)
      audioClip.loop();
    else
    if (audioStart==SOUND_EVENT){    //Start event triggered sound if available and triggered
      JaveMovieFrame currentFrame=movieFile.getFrame(currentFrameIndex);
      if (currentFrame.isSoundTrigger())
        audioClip.play();
    }      
  }

  thread.start();
}

public void doPause(){
  pause=!pause;
  if (thread!=null){
    try{
      thread.interrupt();
    }catch (SecurityException e){
    }
  }
}

public void doStop(){
  if (thread!=null){
    shallStop=true;
    try{
      thread.interrupt();
      while (thread!=null && thread.isAlive())
        Thread.yield();
    }catch (Exception e){
    }
    
    if (audioClip!=null)
      audioClip.stop();
  }
}

public void doPrev(){
  if (!reverse){
    if (currentFrameIndex>0)
      setFrame(currentFrameIndex-1);
    else
    if (loop)
      setFrame(movieFile.getFrameCount()-1);
  }else{
    if (currentFrameIndex<movieFile.getFrameCount()-1)
      setFrame(currentFrameIndex+1);
    else
    if (loop)
      setFrame(0);
  }
}

public void doNext(){
  if (reverse){
    if (currentFrameIndex>0)
      setFrame(currentFrameIndex-1);
    else
    if (loop)
      setFrame(movieFile.getFrameCount()-1);
  }else{
    if (currentFrameIndex<movieFile.getFrameCount()-1)
      setFrame(currentFrameIndex+1);
    else
    if (loop)
      setFrame(0);
    
    //Start event triggered sound if available
    if (audioClip!=null && audioStart==SOUND_EVENT){
      JaveMovieFrame currentFrame=movieFile.getFrame(currentFrameIndex);
      if (currentFrame.isSoundTrigger())
        audioClip.play();
    }      
  }
}

public void doZoomIn(){
  plate.zoomIn();
}

public void doZoomOut(){
  plate.zoomOut();
}

protected Component createStatusBar(){
  Panel p=new Panel();
  tfTool=new TextField(12);  
  tfAction=new TextField(12);  
  tfFrame=new TextField(10); 
  
  p.add(new Label("Frame:", Label.RIGHT));  
  p.add(tfFrame);
  p.add(new Label("Tool:", Label.RIGHT));  
  p.add(tfTool);
  p.add(new Label("Action:", Label.RIGHT));  
  p.add(tfAction);
  
  lFps=new Label("0.00 fps");
  p.add(lFps);
  
  return p;
}


protected Component createControlBar(){
  Panel p=new Panel();
  
  bPrev =new Button("Prev");
  bPrev.addActionListener(this);
  bPlay =new Button("Play");
  bPlay.addActionListener(this);
  bPause=new Button("Pause");
  bPause.addActionListener(this);
  bStop =new Button("Stop");
  bStop.addActionListener(this);
  bNext =new Button("Next");
  bNext.addActionListener(this);
  
  
  cbReverse=new Checkbox("Reverse",false);
  cbReverse.addItemListener(this);
  reverse=false; 

  bZoomIn =new Button("Zoom in");
  bZoomIn.addActionListener(this);
  
  bZoomOut=new Button("Zoom out");
  bZoomOut.addActionListener(this);
  
  chSpeed=new Choice();
  for (int i=0;i<STR_SPEEDS.length;++i)
    chSpeed.add(STR_SPEEDS[i]);
  chSpeed.select(DEFAULT_SPEED);
  chSpeed.addItemListener(this);
  durationMultiplicator=DURATION_MULTIPLICATOR[chSpeed.getSelectedIndex()]; 
  p.add(bPrev);
  p.add(bPlay);
  p.add(bPause);
  p.add(bStop);
  p.add(bNext);
  p.add(cbReverse);
  p.add(chSpeed);

  p.add(bZoomIn);
  p.add(bZoomOut);
  
  return p;
}



public void itemStateChanged(ItemEvent evt){
  Object source=evt.getSource();
  if (source==chSpeed)
    durationMultiplicator=DURATION_MULTIPLICATOR[chSpeed.getSelectedIndex()]; 
  else
  if (source==cbReverse)
    reverse=cbReverse.getState();
}


public void actionPerformed(ActionEvent evt){
  Object source=evt.getSource();
  if (source==bNext){
    doNext();
    return;
  }
  if (source==bPrev){
    doPrev();
    return;
  }
  if (source==bStop){
    doStop();
    return;
  }
  if (source==bPlay){
    doPlay();
    return;
  }
  if (source==bPause){
    doPause();
    return;
  }
  if (source==bZoomIn){
    doZoomIn();
    return;
  }
  if (source==bZoomOut){
    doZoomOut();
    return;
  }
}
  
//WindowListener
public void windowOpened(WindowEvent e){}
public void windowClosing(WindowEvent e){
  System.exit(0);
}
public void windowClosed(WindowEvent e)      {}
public void windowIconified(WindowEvent e)   {}
public void windowDeiconified(WindowEvent e) {}
public void windowActivated(WindowEvent e)   {}
public void windowDeactivated(WindowEvent e){}

/* The Player listens to Mouse Events on the JavePlate
A click on the plate will pause/play the movie
*/
public void mouseClicked(MouseEvent evt){
  if (thread==null)
    doPlay();
  else
    doPause();
}
public void mousePressed(MouseEvent evt){}
public void mouseReleased(MouseEvent evt){}
public void mouseEntered(MouseEvent evt){}
public void mouseExited(MouseEvent evt){}








/* Just for me to test easily - Markus*/
protected final static String DEFAULT_NAME1="e:\\jave\\player\\demo\\tshilp.zip";
protected final static String DEFAULT_NAME2="e:\\jave\\player\\demo\\king_lion.zip";
protected final static String DEFAULT_NAME3="e:\\jave\\player\\demo\\mandel.zip";
protected final static String DEFAULT_NAME="e:\\test\\sw\\sw1.neuspec.jmov"; 


public static void main(String args[]){
  if (args!=null && args.length>0 && args[0]!=null)
    createPlayerApplication(args[0]);
  else{
//    createPlayerApplication(DEFAULT_NAME1);
//    createPlayerApplication(DEFAULT_NAME);
    createPlayerApplication(DEFAULT_NAME2);
//    createPlayerApplication(DEFAULT_NAME3);
  }
}  
 
protected static void createPlayerApplication(String filename){
  JavePlayer player=new JavePlayer();
   
  Frame frame=new Frame(TITLE);
  frame.setLayout(new GridLayout(1,0,0,0));
  frame.add(player);
  frame.addWindowListener(player);
  frame.pack();
  frame.setSize(frame.getSize().width, 500);  
  frame.show();
  player.load(filename);
}
/* Just for me to test easily - Markus */

}