/*
 * @(#)JavePlayerApplet.java  1.2  23rd December 2001 Markus Gebhard
 * @(#)JavePlayerApplet.java  1.1  11th July 2001 Markus Gebhard
 * @(#)JavePlayerApplet.java  1.0  28th June 2001 Markus Gebhard
 * @(#)JavePlayerApplet.java  0.1  10th June 2001 Markus Gebhard
 * 
 * This Software is Open Source. You are allowed to change anything,
 * but there are some limitations:
 *
 *  - You must always include the source code
 *  - You are not allowed to remove this Header from the source code
 *  - You have to add a link to http://www.jave.de so people know
 *    where to find the original version and information about it
 *  - You have to let me know about new releases
 *
 *                                      Markus Gebhard  markus@jave.de
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.AudioClip;

public class JavePlayerApplet extends java.applet.Applet{

  protected boolean initialized=false;
  protected JavePlayer player;
  
/**
* Applet info.
*/
public String getAppletInfo() {
  return JavePlayer.TITLE;
}

/**
* Parameter info.
*/
public String[][] getParameterInfo() {
  String[][] info = {
    // Parameter Name Type of Value   Description
    {"FILE",          "String",    "Name of the Jave movie file to be loaded"},
    {"BACKGROUND",    "Hex-Value", "Color for background, e.g.: #FFFF60"},
    {"FOREGROUND",    "Hex-Value", "Color for foreground, e.g.: #FFFF60"},
    {"DURATION",      "int",       "Default duration for a frame"},
    {"AUTOSTART",     "int",       "Player will immediately start playing the movie if value is 1"},
    {"LOOP",          "int",       "Player will loop infinite times if value is 1"},
    {"CONTROLS",      "int",       "0: No controls 1: Just buttons 2: All controls visible"},
    {"SOUND",         "String",    "Name of a sound file (*.mid, *.au or *.wav) to be played at play()"},
    {"SOUNDSTART",    "String",    "When to play the sound file: 'once', 'loop' or 'event'"},
  };
  return info;
}

public void init(){
  if (initialized)
    return;
    
  //Get all parameters but the FILE parameter (will be fetched later)
  int defaultDuration = JavePlayer.DEFAULT_DURATION;
  int controls        = JavePlayer.DEFAULT_CONTROLS;
  boolean loop        = JavePlayer.DEFAULT_LOOP;
  boolean autostart   = JavePlayer.DEFAULT_AUTOSTART;
  Color background    = JavePlayer.DEFAULT_BACKGROUND;
  Color foreground    = JavePlayer.DEFAULT_FOREGROUND;
  String soundFile    = null;
  int soundStart      = JavePlayer.DEFAULT_SOUND_START;
  
  try{
    String s=getParameter("DURATION");
    if (s!=null){
      try{
        defaultDuration=Integer.parseInt(s);
      }catch(NumberFormatException e){
        System.err.println("Error reading parameter 'DURATION': "+e);
      }
    }

    s=getParameter("CONTROLS");
    if (s!=null){
      if (s.equals("0"))
        controls=0;
      else
      if (s.equals("1"))
        controls=1;
      else
        System.err.println("Error reading parameter 'CONTROLS': Must be 0 or 1");
    }

    s=getParameter("LOOP");
    if (s!=null && s.equals("1"))
      loop=true;

    s=getParameter("SOUND");
    if (s!=null && s.length()>0)
      soundFile=s;
      
    s=getParameter("SOUNDSTART");
    if (s!=null){
      s=s.toLowerCase();
      if (s.equals("once"))
        soundStart=JavePlayer.SOUND_ONCE;
      else
      if (s.equals("loop"))
        soundStart=JavePlayer.SOUND_LOOP;
      else
      if (s.equals("event"))
        soundStart=JavePlayer.SOUND_EVENT;
      else
        System.err.println("Error reading parameter 'SOUNDSTART': Must be 'once' or 'loop' or 'event'");
    }
 
    s=getParameter("AUTOSTART");
    if (s!=null && s.equals("1"))
      autostart=true;
    
    s=getParameter("BACKGROUND");
    if (s!=null)
      background=JavePlayerTools.hexToColor(s);
    
    s=getParameter("FOREGROUND");
    if (s!=null)
      foreground=JavePlayerTools.hexToColor(s);
  }catch(Exception e){
    //Don't worry about exceptions - defaults will do
    System.err.println("Error reading parameters: "+e);
  }
    
  player=new JavePlayer(defaultDuration, controls, loop, autostart, background, foreground);
  
  //Now load AudioClip if sound is defined by applet parameter
  if (soundFile!=null){
    AudioClip audio = getAudioClip(getDocumentBase(), soundFile);
    player.setAudioClip(audio);
    player.setAudioStart(soundStart);
  }
  
  
  
  setLayout(new GridLayout(1,0,0,0));
  add(player);
  
  initialized=true;
}

public void start(){
  player.load(getCodeBase()+getParameter("FILE"));
}

public void stop(){
}

}